# vim:set ft= ts=4 sw=4 et fdm=marker:
use lib '.';
use t::TestLRUCache;

repeat_each(2);

plan tests => repeat_each() * (blocks() * 3);

no_long_string();
run_tests();

__DATA__

=== TEST 1: capacity() returns total cache capacity
--- config
    location = /t {
        content_by_lua_block {
            local lrucache = require "resty.lrucache.pureffi"
            local c = lrucache.new(2)

            ngx.say("capacity: ", c:capacity())
        }
    }
--- response_body
capacity: 2
